@@ -25,6 +25,9 @@
 #ifndef KHTMLFONT_H
 #define KHTMLFONT_H
 
+#include <khtml_global.h>
+#include <khtml_settings.h>
+
 #include <QCache>
 #include <QtGui/QFont>
 #include <QtGui/QFontMetrics>
@@ -79,6 +82,8 @@ public:
     int descent;
     int height;
     int lineSpacing;
+    int xHeight;
+    int m_zeroCharWidth;
     mutable bool invalidated;
 
     ~CachedFontInstance();
@@ -104,7 +109,7 @@ class FontDef
 {
 public:
     FontDef()
-        : size( 0 ), italic( false ), smallCaps( false ), weight( QFont::Normal/*50*/ ) {}
+        : family(KHTMLGlobal::defaultHTMLSettings()->stdFontName()), size(0), italic(false), smallCaps(false), weight(QFont::Normal) {}
     bool operator == ( const FontDef &other ) const {
         return ( family == other.family &&
                  size == other.size &&
@@ -175,7 +180,7 @@ public:
      *		decoration painting
      * @param deco combined text decoration (see Decoration)
      */
-    void drawText( QPainter *p, int x, int y, const QChar *str, int slen, int pos, int len, int width,
+    void drawText( QPainter *p, int x, int y, const QChar *str, const int slen, int pos, int len, int width,
                    Qt::LayoutDirection d, int from=-1, int to=-1, QColor bg=QColor(),
 		   int uy=-1, int h=-1, int deco=0 ) const;
 
@@ -237,7 +242,10 @@ public:
     int descent() const { return cfi->descent; }
     int height() const { return cfi->height; }
     int lineSpacing() const { return cfi->lineSpacing; }
-    float xHeight() const { return cfi->fm.xHeight(); }
+    int xHeight() const { return cfi->xHeight; }
+
+    // return -1 if '0' char glyph not in font
+    int zeroCharWidth() const { return cfi->m_zeroCharWidth; }
     //FIXME: IMPLEMENT ME
     unsigned unitsPerEm() const { return 0; }
     int spaceWidth() const { return 0; }
